@@ -1,8 +1,7 @@
 import os
 
-SCHEMA_FILE = os.path.abspath(os.path.join(
-    os.path.dirname(os.path.dirname(__file__)),
-    'schemas.json'))
+# Destination directory
+TARGET_DIR = os.path.abspath('results')
 
 # Addons Server defaults
 ADDONS_SERVER = 'https://addons-dev.allizom.org/'
@@ -10,7 +9,7 @@ ADDONS_SERVER = 'https://addons-dev.allizom.org/'
 # Kinto server defaults
 KINTO_SERVER = 'http://localhost:8888/v1'
 AUTH = ('mark', 'p4ssw0rd')
-COLLECTION_PERMISSIONS = {'read': ["system.Everyone"]}
+COLLECTION_PERMISSIONS = {}
 
 # Buckets name default
 ADDONS_BUCKET = u'staging'
@@ -22,6 +21,7 @@ GFX_COLLECTION = u'gfx'
 PLUGINS_BUCKET = u'staging'
 PLUGINS_COLLECTION = u'plugins'
 
+DESTINATION_BUCKET = 'blocklists'
 
 # APP ID
 FIREFOX_APPID = '{ec8030f7-c20a-464f-9b0e-13a3a9e97384}'
