@@ -1,7 +1,7 @@
 import sys
 from collections import OrderedDict
 from lxml import etree
-from kinto_client import cli_utils
+from kinto_http import cli_utils
 from . import constants
 from .compare import version_int
 
@@ -39,27 +39,53 @@ def build_version_range(root, item, app_id, app_ver=None):
                             id=tA['guid'])
                         etree.SubElement(
                             targetApplication, 'versionRange',
-                            minVersion=tA['minVersion'],
-                            maxVersion=tA['maxVersion'])
+                            maxVersion=tA['maxVersion'],
+                            minVersion=tA['minVersion'])
 
 
-def is_related_to(item, app_id):
-    """Return True if the item relates to the given app_id."""
-    if not item.get('versionRange'):
+def is_related_to(item, app_id, app_ver=None):
+    """Return True if the item relates to the given app_id (and app_ver, if passed)."""
+    versionRange = item.get('versionRange')
+    if not versionRange:
         return True
 
-    for vR in item['versionRange']:
+    for vR in versionRange:
         if not vR.get('targetApplication'):
             return True
-
-        for tA in vR['targetApplication']:
-            if tA['guid'] == app_id:
-                return True
-
+        if get_related_targetApplication(vR, app_id, app_ver) is not None:
+            return True
     return False
 
 
-def write_addons_items(xml_tree, records, app_id, api_ver=3):
+def get_related_targetApplication(vR, app_id, app_ver):
+    """Return the first matching target application in this version range.
+    Returns None if there are no target applications or no matching ones."""
+    targetApplication = vR.get('targetApplication')
+    if not targetApplication:
+        return None
+
+    for tA in targetApplication:
+        guid = tA.get('guid')
+        if not guid or guid == app_id:
+            if not app_ver:
+                return tA
+            # We purposefully use maxVersion only, so that the blocklist contains items
+            # whose minimum version is ahead of the version we get passed. This means
+            # the blocklist we serve is "future-proof" for app upgrades.
+            if between(version_int(app_ver), '0', tA.get('maxVersion', '*')):
+                return tA
+
+    return None
+
+
+def should_include_certs(app_id=None, app_ver=None):
+    """Whether we should include certificate blocklist items for this application version."""
+    # As of Firefox 58, we definitely won't need the cert items in here.
+    # They get fetched via a different means than the XML-based blocklist
+    return app_id != constants.FIREFOX_APPID or version_int(app_ver) < version_int("58.0")
+
+
+def write_addons_items(xml_tree, records, app_id, api_ver=3, app_ver=None):
     """Generate the addons blocklists.
 
     <emItem blockID="i372" id="5nc3QHFgcb@r06Ws9gvNNVRfH.com">
@@ -78,23 +104,42 @@ def write_addons_items(xml_tree, records, app_id, api_ver=3):
         return
 
     emItems = etree.SubElement(xml_tree, 'emItems')
+    groupby = {}
     for item in records:
-        if is_related_to(item, app_id):
-            emItem = etree.SubElement(emItems, 'emItem',
-                                      blockID=item.get('blockID', item['id']))
+        if is_related_to(item, app_id, app_ver):
+            if item['guid'] in groupby:
+                emItem = groupby[item['guid']]
+                # When creating new records from the Kinto Admin we don't have proper blockID.
+                if 'blockID' in item:
+                    # Remove the first caracter which is the letter i to
+                    # compare the numeric value i45 < i356.
+                    current_blockID = int(item['blockID'][1:])
+                    previous_blockID = int(emItem.attrib['blockID'][1:])
+                    # Group by and keep the biggest blockID in the XML file.
+                    if current_blockID > previous_blockID:
+                        emItem.attrib['blockID'] = item['blockID']
+                else:
+                    # If the latest entry does not have any blockID attribute, its
+                    # ID should be used. (the list of records is sorted by ascending
+                    # last_modified).
+                    # See https://bugzilla.mozilla.org/show_bug.cgi?id=1473194
+                    emItem.attrib['blockID'] = item['id']
+            else:
+                emItem = etree.SubElement(emItems, 'emItem',
+                                          blockID=item.get('blockID', item['id']))
+                groupby[item['guid']] = emItem
+                prefs = etree.SubElement(emItem, 'prefs')
+                for p in item.get('prefs', []):
+                    pref = etree.SubElement(prefs, 'pref')
+                    pref.text = p
+
+            # Set the add-on ID
+            emItem.set('id', item['guid'])
 
             for field in ['name', 'os']:
                 if field in item:
                     emItem.set(field, item[field])
 
-            if 'guid' in item:
-                emItem.set('id', item['guid'])
-
-            prefs = etree.SubElement(emItem, 'prefs')
-            for p in item['prefs']:
-                pref = etree.SubElement(prefs, 'pref')
-                pref.text = p
-
             build_version_range(emItem, item, app_id)
 
 
@@ -108,7 +153,7 @@ def write_plugin_items(xml_tree, records, app_id, api_ver=3, app_ver=None):
     """Generate the plugin blocklists.
 
     <pluginItem blockID="p422">
-        <match name="filename" exp="JavaAppletPlugin\.plugin"/>
+        <match name="filename" exp="JavaAppletPlugin\\.plugin"/>
         <versionRange minVersion="Java 7 Update 16"
                       maxVersion="Java 7 Update 24"
                       severity="0" vulnerabilitystatus="1">
@@ -124,35 +169,17 @@ def write_plugin_items(xml_tree, records, app_id, api_ver=3, app_ver=None):
 
     pluginItems = etree.SubElement(xml_tree, 'pluginItems')
     for item in records:
-        if is_related_to(item, app_id):
-            for versionRange in item.get('versionRange', []):
-                if not versionRange.get('targetApplication'):
-                    add_plugin_item(pluginItems, item, versionRange,
+        for versionRange in item.get('versionRange', []):
+            if not versionRange.get('targetApplication'):
+                add_plugin_item(pluginItems, item, versionRange,
+                                app_id=app_id, api_ver=api_ver,
+                                app_ver=app_ver)
+            else:
+                targetApplication = get_related_targetApplication(versionRange, app_id, app_ver)
+                if targetApplication is not None:
+                    add_plugin_item(pluginItems, item, versionRange, targetApplication,
                                     app_id=app_id, api_ver=api_ver,
                                     app_ver=app_ver)
-                else:
-                    for targetApplication in versionRange['targetApplication']:
-                        is_targetApplication_related = (
-                            'guid' not in targetApplication or
-                            targetApplication['guid'] == app_id
-                        )
-                        if is_targetApplication_related:
-                            if api_ver < 3 and app_ver is not None:
-                                app_version = version_int(app_ver)
-                                is_version_related = between(
-                                    app_version,
-                                    targetApplication.get('minVersion', 0),
-                                    targetApplication.get('maxVersion', '*'))
-                                if is_version_related:
-                                    add_plugin_item(
-                                        pluginItems, item, versionRange,
-                                        targetApplication, app_id=app_id,
-                                        api_ver=api_ver, app_ver=app_ver)
-                            else:
-                                add_plugin_item(
-                                    pluginItems, item, versionRange,
-                                    targetApplication, app_id=app_id,
-                                    api_ver=api_ver, app_ver=app_ver)
 
 
 def add_plugin_item(pluginItems, item, version, tA=None, app_id=None,
@@ -169,8 +196,8 @@ def add_plugin_item(pluginItems, item, version, tA=None, app_id=None,
         json_field = 'match%s' % xml_field.capitalize()
         if json_field in item:
             etree.SubElement(entry, 'match',
-                             name=xml_field,
-                             exp=item[json_field])
+                             exp=item[json_field],
+                             name=xml_field,)
 
     if 'infoURL' in item:
         infoURL = etree.SubElement(entry, 'infoURL')
@@ -242,8 +269,8 @@ def add_plugin_item(pluginItems, item, version, tA=None, app_id=None,
             id=tA['guid'])
         etree.SubElement(targetApplication,
                          'versionRange',
-                         minVersion=tA['minVersion'],
-                         maxVersion=tA['maxVersion'])
+                         maxVersion=tA['maxVersion'],
+                         minVersion=tA['minVersion'])
 
 
 def write_gfx_items(xml_tree, records, app_id, api_ver=3):
@@ -259,6 +286,7 @@ def write_gfx_items(xml_tree, records, app_id, api_ver=3):
         <featureStatus>BLOCKED_DRIVER_VERSION</featureStatus>
         <driverVersion>8.17.12.5896</driverVersion>
         <driverVersionComparator>LESS_THAN_OR_EQUAL</driverVersionComparator>
+        <versionRange minVersion="3.2" maxVersion="3.4" />
     </gfxBlacklistEntry>
     """
     if not records:
@@ -285,23 +313,43 @@ def write_gfx_items(xml_tree, records, app_id, api_ver=3):
                     device = etree.SubElement(devices, 'device')
                     device.text = d
 
+            if 'versionRange' in item:
+                version = item['versionRange']
+                versionRange = etree.SubElement(entry, 'versionRange')
 
-def write_cert_items(xml_tree, records, api_ver=3):
+                for field in ['minVersion', 'maxVersion']:
+                    value = version.get(field)
+                    if value:
+                        versionRange.set(field, str(value))
+
+
+def write_cert_items(xml_tree, records, api_ver=3, app_id=None, app_ver=None):
     """Generate the certificate blocklists.
 
     <certItem issuerName="MIGQMQswCQYD...IENB">
       <serialNumber>UoRGnb96CUDTxIqVry6LBg==</serialNumber>
     </certItem>
+
+    or
+
+    <certItem subject='MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5'
+              pubKeyHash='VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8='>
+    </certItem>
     """
-    if not records:
+    if not records or not should_include_certs(app_id, app_ver):
         return
 
     certItems = etree.SubElement(xml_tree, 'certItems')
     for item in records:
-        cert = etree.SubElement(certItems, 'certItem',
-                                issuerName=item['issuerName'])
-        serialNumber = etree.SubElement(cert, 'serialNumber')
-        serialNumber.text = item['serialNumber']
+        if item.get('subject') and item.get('pubKeyHash'):
+            cert = etree.SubElement(certItems, 'certItem',
+                                    pubKeyHash=item['pubKeyHash'],
+                                    subject=item['subject'])
+        else:
+            cert = etree.SubElement(certItems, 'certItem',
+                                    issuerName=item['issuerName'])
+            serialNumber = etree.SubElement(cert, 'serialNumber')
+            serialNumber.text = item['serialNumber']
 
 
 def main(args=None):
@@ -343,9 +391,15 @@ def main(args=None):
                         help='Collection name for plugin',
                         type=str, default=constants.PLUGINS_COLLECTION)
 
+    parser.add_argument('--api-version', help='Targeted blocklists.xml APP id',
+                        type=int, default=3)
+
     parser.add_argument('--app', help='Targeted blocklists.xml APP id',
                         type=str, default=constants.FIREFOX_APPID)
 
+    parser.add_argument('--app-version', help='The targetted app version',
+                        type=str, default=None)
+
     # Choose where to write the file down.
     parser.add_argument('-o', '--out', help='Output XML file.',
                         type=str, default=None)
@@ -371,7 +425,7 @@ def main(args=None):
             collection=args.addons_collection,
             enabled=True,
             _sort="last_modified")
-    except:
+    except Exception:
         logger.warn(
             'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
                 bucket=args.addons_bucket,
@@ -389,7 +443,7 @@ def main(args=None):
             collection=args.plugins_collection,
             enabled=True,
             _sort="last_modified")
-    except:
+    except Exception:
         logger.warn(
             'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
                 bucket=args.plugins_bucket,
@@ -407,7 +461,7 @@ def main(args=None):
             collection=args.gfx_collection,
             enabled=True,
             _sort="last_modified")
-    except:
+    except Exception:
         logger.warn(
             'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
                 bucket=args.gfx_bucket,
@@ -419,34 +473,47 @@ def main(args=None):
     if gfx_records:
         last_update = max(last_update, gfx_records[-1]['last_modified'])
 
-    try:
-        cert_records = client.get_records(
-            bucket=args.certificates_bucket,
-            collection=args.certificates_collection,
-            enabled=True,
-            _sort="last_modified")
-    except:
-        logger.warn(
-            'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
+    cert_records = None
+    if should_include_certs(args.app, args.app_version):
+        try:
+            cert_records = client.get_records(
                 bucket=args.certificates_bucket,
                 collection=args.certificates_collection,
+                enabled=True,
+                _sort="last_modified")
+        except Exception:
+            logger.warn(
+                'Unable to fetch the ``{bucket}/{collection}`` records.'.format(
+                    bucket=args.certificates_bucket,
+                    collection=args.certificates_collection,
+                )
             )
-        )
-        cert_records = []
+            cert_records = []
 
-    if cert_records:
-        last_update = max(last_update, cert_records[-1]['last_modified'])
+        if cert_records:
+            last_update = max(last_update, cert_records[-1]['last_modified'])
 
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='%s' % last_update,
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='%s' % last_update
     )
 
-    write_addons_items(xml_tree, addons_records, app_id=args.app)
-    write_plugin_items(xml_tree, plugin_records, app_id=args.app)
-    write_gfx_items(xml_tree, gfx_records, app_id=args.app)
-    write_cert_items(xml_tree, cert_records)
+    write_addons_items(xml_tree, addons_records,
+                       api_ver=args.api_version,
+                       app_id=args.app,
+                       app_ver=args.app_version)
+    write_plugin_items(xml_tree, plugin_records,
+                       api_ver=args.api_version,
+                       app_id=args.app,
+                       app_ver=args.app_version)
+    write_gfx_items(xml_tree, gfx_records,
+                    api_ver=args.api_version,
+                    app_id=args.app)
+    write_cert_items(xml_tree, cert_records,
+                     api_ver=args.api_version,
+                     app_id=args.app,
+                     app_ver=args.app_version)
 
     doc = etree.ElementTree(xml_tree)
     out_fd.write(etree.tostring(
