@@ -1,7 +1,7 @@
 import mock
 import unittest
+from io import StringIO
 from lxml import etree
-from six import StringIO
 
 from amo2kinto import constants, exporter, compare
 
@@ -20,7 +20,7 @@ ADDONS_DATA = {
     },
     "versionRange": [{
         "targetApplication": [
-            {"guid": "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
+            {"guid": constants.FIREFOX_APPID,
              "minVersion": "3.6",
              "maxVersion": "3.6.*"},
             {"guid": "{some-other-application}",
@@ -52,8 +52,8 @@ ADDONS_DATA = {
 def test_addon_record():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     exporter.write_addons_items(xml_tree, [ADDONS_DATA],
@@ -85,11 +85,49 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 """.decode('utf-8')
 
 
+def test_addon_record_with_no_blockID():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    data = ADDONS_DATA.copy()
+    del data['blockID']
+    exporter.write_addons_items(xml_tree, [data],
+                                constants.FIREFOX_APPID)
+
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist">
+  <emItems>
+    <emItem blockID="e3e8f123-588d-0f73-63d8-93bdfc6ae8e2" id="sqlmoz@facebook.com">
+      <prefs>
+        <pref>test.blocklist</pref>
+      </prefs>
+      <versionRange minVersion="0" maxVersion="*" severity="3">
+        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
+          <versionRange maxVersion="3.6.*" minVersion="3.6"/>
+        </targetApplication>
+      </versionRange>
+      <versionRange minVersion="0" maxVersion="*"/>
+    </emItem>
+  </emItems>
+</blocklist>
+""".decode('utf-8')
+
+
 def test_addon_record_with_no_version_range_info():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = ADDONS_DATA.copy()
@@ -121,8 +159,8 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 def test_addon_record_with_no_targetApplication_info():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = ADDONS_DATA.copy()
@@ -148,8 +186,8 @@ def test_addon_record_with_no_targetApplication_info():
 <blocklist lastupdate="1459262434336" \
 xmlns="http://www.mozilla.org/2006/addons-blocklist">
   <emItems>
-    <emItem blockID="i454" name="Mozilla Service Pack (malware)" \
-os="WINNT" id="sqlmoz@facebook.com">
+    <emItem blockID="i454" id="sqlmoz@facebook.com" \
+name="Mozilla Service Pack (malware)" os="WINNT">
       <prefs>
         <pref>test.blocklist</pref>
       </prefs>
@@ -163,8 +201,8 @@ os="WINNT" id="sqlmoz@facebook.com">
 def test_addon_record_with_no_targetApplication_matching():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = ADDONS_DATA.copy()
@@ -196,6 +234,211 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 """.decode('utf-8')
 
 
+def test_addon_record_with_no_targetApplication_version_matching():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    data = ADDONS_DATA.copy()
+    data['versionRange'] = [{
+        "targetApplication": [
+            {"guid": constants.FIREFOX_APPID,
+             "minVersion": "3.6",
+             "maxVersion": "3.6.*"}
+        ],
+        "minVersion": "0",
+        "maxVersion": "*",
+        "severity": 3
+    }]
+
+    exporter.write_addons_items(xml_tree, [data],
+                                constants.FIREFOX_APPID, app_ver="3.7")
+
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist">
+  <emItems/>
+</blocklist>
+""".decode('utf-8')
+
+
+def test_addon_record_with_future_targetApplication_version_matching():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    exporter.write_addons_items(xml_tree, [ADDONS_DATA],
+                                constants.FIREFOX_APPID, app_ver="3.1")
+
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist">
+  <emItems>
+    <emItem blockID="i454" id="sqlmoz@facebook.com">
+      <prefs>
+        <pref>test.blocklist</pref>
+      </prefs>
+      <versionRange minVersion="0" maxVersion="*" severity="3">
+        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
+          <versionRange maxVersion="3.6.*" minVersion="3.6"/>
+        </targetApplication>
+      </versionRange>
+      <versionRange minVersion="0" maxVersion="*"/>
+    </emItem>
+  </emItems>
+</blocklist>
+""".decode('utf-8')
+
+
+def test_addon_record_group_by_addon_id():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    data = ADDONS_DATA.copy()
+    data['blockID'] = 'i6'
+    data['versionRange'] = [{
+        "targetApplication": [
+            {"guid": constants.FIREFOX_APPID,
+             "minVersion": "1.2",
+             "maxVersion": "1.4"}
+        ],
+        "minVersion": "0",
+        "maxVersion": "*",
+        "severity": 3
+    }]
+
+    data2 = ADDONS_DATA.copy()
+    data2['blockID'] = 'i586'
+    data2['versionRange'] = [{
+        "targetApplication": [
+            {"guid": constants.FIREFOX_APPID,
+             "minVersion": "3.6",
+             "maxVersion": "3.6.*"}
+        ],
+        "minVersion": "0",
+        "maxVersion": "*",
+        "severity": 2
+    }]
+
+    exporter.write_addons_items(xml_tree, [data, data2],
+                                constants.FIREFOX_APPID)
+
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist">
+  <emItems>
+    <emItem blockID="i586" id="sqlmoz@facebook.com">
+      <prefs>
+        <pref>test.blocklist</pref>
+      </prefs>
+      <versionRange minVersion="0" maxVersion="*" severity="3">
+        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
+          <versionRange maxVersion="1.4" minVersion="1.2"/>
+        </targetApplication>
+      </versionRange>
+      <versionRange minVersion="0" maxVersion="*" severity="2">
+        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
+          <versionRange maxVersion="3.6.*" minVersion="3.6"/>
+        </targetApplication>
+      </versionRange>
+    </emItem>
+  </emItems>
+</blocklist>
+""".decode('utf-8')
+
+
+def test_addon_record_group_by_addon_id_without_blockID():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    data = ADDONS_DATA.copy()
+    data['blockID'] = 'i60'
+    data['versionRange'] = [{
+        "targetApplication": [
+            {"guid": constants.FIREFOX_APPID,
+             "minVersion": "1.2",
+             "maxVersion": "1.4"}
+        ],
+        "minVersion": "0",
+        "maxVersion": "*",
+        "severity": 3
+    }]
+
+    data2 = ADDONS_DATA.copy()
+    del data2['blockID']
+    data2['id'] = '44faef61-a1dd-4c4a-bc8a-cbbf586adff3'
+    data2['versionRange'] = [{
+        "targetApplication": [
+            {"guid": constants.FIREFOX_APPID,
+             "minVersion": "3.6",
+             "maxVersion": "3.6.*"}
+        ],
+        "minVersion": "0",
+        "maxVersion": "*",
+        "severity": 2
+    }]
+
+    exporter.write_addons_items(xml_tree, [data, data2],
+                                constants.FIREFOX_APPID)
+
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist">
+  <emItems>
+    <emItem blockID="44faef61-a1dd-4c4a-bc8a-cbbf586adff3" id="sqlmoz@facebook.com">
+      <prefs>
+        <pref>test.blocklist</pref>
+      </prefs>
+      <versionRange minVersion="0" maxVersion="*" severity="3">
+        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
+          <versionRange maxVersion="1.4" minVersion="1.2"/>
+        </targetApplication>
+      </versionRange>
+      <versionRange minVersion="0" maxVersion="*" severity="2">
+        <targetApplication id="{ec8030f7-c20a-464f-9b0e-13a3a9e97384}">
+          <versionRange maxVersion="3.6.*" minVersion="3.6"/>
+        </targetApplication>
+      </versionRange>
+    </emItem>
+  </emItems>
+</blocklist>
+""".decode('utf-8')
+
+
 PLUGIN_DATA = {
     "id": "6a1b6dfe-f463-3061-e8f8-6e896ccf2a8a",
     "last_modified": "1301581706645",
@@ -221,7 +464,7 @@ PLUGIN_DATA = {
         "vulnerabilityStatus": 1,
         "targetApplication": [{
             "minVersion": "3.0a1",
-            "guid": "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
+            "guid": constants.FIREFOX_APPID,
             "maxVersion": "3.*"
         }]
     }]
@@ -232,11 +475,15 @@ def test_between_does_not_fails_with_none():
     assert exporter.between(compare.version_int("46.2"), None, None)
 
 
+def test_get_related_targetApplication_accepts_versionRange_without_targetApplication():
+    assert exporter.get_related_targetApplication({}, None, None) is None
+
+
 def test_plugin_record():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     exporter.write_plugin_items(xml_tree, [PLUGIN_DATA],
@@ -272,8 +519,8 @@ vulnerabilitystatus="1">
 def test_plugin_record_with_api_version_2():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     exporter.write_plugin_items(xml_tree, [PLUGIN_DATA],
@@ -306,8 +553,8 @@ vulnerabilitystatus="1"/>
 def test_plugin_record_with_api_version_2_with_no_guid():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
@@ -348,8 +595,8 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 def test_plugin_record_with_api_version_2_with_no_guid_and_no_vulnerability():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
@@ -388,8 +635,8 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 def test_plugin_record_with_api_version_2_with_no_guid_and_severity_only():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
@@ -427,8 +674,8 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 def test_plugin_record_with_api_version_2_with_no_guid_and_severity_0():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
@@ -466,14 +713,14 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 def test_plugin_record_with_api_version_2_with_guid_and_no_min_max_version():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
     data['versionRange'] = [{
         "targetApplication": [
-            {"guid": "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
+            {"guid": constants.FIREFOX_APPID,
              "minVersion": "3.6",
              "maxVersion": "3.6.*"}
         ],
@@ -510,14 +757,14 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 def test_plugin_record_with_api_version_2_with_guid_and_empty_versionRange():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
     data['versionRange'] = [{
         "targetApplication": [
-            {"guid": "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
+            {"guid": constants.FIREFOX_APPID,
              "minVersion": "3.6",
              "maxVersion": "3.6.*"}
         ]
@@ -552,14 +799,14 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 def test_plugin_record_with_api_version_2_with_related_version():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
     data['versionRange'] = [{
         "targetApplication": [
-            {"guid": "{ec8030f7-c20a-464f-9b0e-13a3a9e97384}",
+            {"guid": constants.FIREFOX_APPID,
              "minVersion": "3.6",
              "maxVersion": "3.6.*"}
         ],
@@ -596,8 +843,8 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 def test_plugin_record_with_no_targetApplication_info():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
@@ -642,8 +889,8 @@ vulnerabilitystatus="1"/>
 def test_plugin_record_with_no_targetApplication_matching():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     data = PLUGIN_DATA.copy()
@@ -685,6 +932,10 @@ GFX_DATA = {
     "feature": "DIRECT2D",
     "featureStatus": "BLOCKED_DRIVER_VERSION",
     "vendor": "0x10de",
+    "versionRange": {
+        "minVersion": "3.2",
+        "maxVersion": "3.4"
+    },
     "details": {
         "who": "All Firefox users who have these drivers installed.",
         "created": "2012-09-24T08:23:33Z",
@@ -700,8 +951,8 @@ GFX_DATA = {
 def test_gfx_record():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     exporter.write_gfx_items(xml_tree, [GFX_DATA],
@@ -727,6 +978,7 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
       <devices>
         <device>0x0a6c</device>
       </devices>
+      <versionRange minVersion="3.2" maxVersion="3.4"/>
     </gfxBlacklistEntry>
   </gfxItems>
 </blocklist>
@@ -752,8 +1004,8 @@ CERTIFICATE_DATA = {
 def test_certificate_record():
     xml_tree = etree.Element(
         'blocklist',
+        lastupdate='1459262434336',
         xmlns="http://www.mozilla.org/2006/addons-blocklist",
-        lastupdate='1459262434336'
     )
 
     exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA])
@@ -776,9 +1028,161 @@ xmlns="http://www.mozilla.org/2006/addons-blocklist">
 """.decode('utf-8')
 
 
+CERTIFICATE_DATA_WITH_EMPTY_PUB_KEY = {
+    "id": "fe7681eb-8480-718e-9870-084dca698f1d",
+    "last_modified": "1448372434324",
+    "blockID": "c796",
+    "issuerName": "MBQxEjAQBgNVBAMTCWVEZWxsUm9vdA==",
+    "serialNumber": "a8V7lRiTqpdLYkrAiPw7tg==",
+    "subject": "",
+    "pubKeyHash": "=",
+    "details": {
+        "who": ".",
+        "created": "2015-11-24T14:40:34Z",
+        "name": "eDellRoot",
+        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1227729",
+        "why": "."
+    }
+}
+
+
+def test_certificate_record_with_empty_pub_key():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA_WITH_EMPTY_PUB_KEY])
+
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist">
+  <certItems>
+    <certItem issuerName="MBQxEjAQBgNVBAMTCWVEZWxsUm9vdA==">
+      <serialNumber>a8V7lRiTqpdLYkrAiPw7tg==</serialNumber>
+    </certItem>
+  </certItems>
+</blocklist>
+""".decode('utf-8')
+
+
+CERTIFICATE_DATA_WITH_SUBJECT = {
+    "id": "fe7681eb-8480-718e-9870-084dca698f1d",
+    "last_modified": "1448372434324",
+    "blockID": "c796",
+    "subject": "MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5",
+    "pubKeyHash": "VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8=",
+    "details": {
+        "who": ".",
+        "created": "2017-10-02T14:23:34Z",
+        "name": "Test End-entity",
+        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1404915",
+        "why": "."
+    }
+}
+
+
+def test_certificate_record_with_subject_and_pubKeyHash():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA_WITH_SUBJECT])
+
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist">
+  <certItems>
+    <certItem pubKeyHash="VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8=" \
+subject="MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5"/>
+  </certItems>
+</blocklist>
+""".decode('utf-8')
+
+
+CERTIFICATE_DATA_WITH_SUBJECT_AND_EMPTY_ISSUER = {
+    "id": "fe7681eb-8480-718e-9870-084dca698f1d",
+    "last_modified": "1448372434324",
+    "blockID": "c796",
+    "subject": "MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5",
+    "pubKeyHash": "VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8=",
+    "issuerName": "",
+    "serialNumber": "",
+    "details": {
+        "who": ".",
+        "created": "2017-10-02T14:23:34Z",
+        "name": "Test End-entity",
+        "bug": "https://bugzilla.mozilla.org/show_bug.cgi?id=1404915",
+        "why": "."
+    }
+}
+
+
+def test_certificate_record_with_subject_and_pubKeyHash_with_empty_issuerName():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA_WITH_SUBJECT_AND_EMPTY_ISSUER])
+
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist">
+  <certItems>
+    <certItem pubKeyHash="VCIlmPM9NkgFQtrs4Oa5TeFcDu6MWRTKSNdePEhOgD8=" \
+subject="MCIxIDAeBgNVBAMMF0Fub3RoZXIgVGVzdCBFbmQtZW50aXR5"/>
+  </certItems>
+</blocklist>
+""".decode('utf-8')
+
+
+def test_certificate_record_post_fx58():
+    xml_tree = etree.Element(
+        'blocklist',
+        lastupdate='1459262434336',
+        xmlns="http://www.mozilla.org/2006/addons-blocklist",
+    )
+
+    exporter.write_cert_items(xml_tree, [CERTIFICATE_DATA_WITH_SUBJECT_AND_EMPTY_ISSUER],
+                              app_id=constants.FIREFOX_APPID, app_ver="59.0")
+    result = etree.tostring(
+        etree.ElementTree(xml_tree),
+        pretty_print=True,
+        xml_declaration=True,
+        encoding='UTF-8').decode('utf-8')
+
+    assert result == b"""<?xml version='1.0' encoding='UTF-8'?>
+<blocklist lastupdate="1459262434336" \
+xmlns="http://www.mozilla.org/2006/addons-blocklist"/>
+""".decode('utf-8')
+
+
 class TestMain(unittest.TestCase):
     def setUp(self):
-        p = mock.patch('kinto_client.cli_utils.Client')
+        p = mock.patch('kinto_http.cli_utils.Client')
         self.MockedClient = p.start()
         self.MockedClient.return_value.get_records.return_value = []
         self.addCleanup(p.stop)
@@ -798,7 +1202,10 @@ class TestMain(unittest.TestCase):
         kinto_client.assert_called_with(server_url=kwargs['kinto_server'],
                                         auth=kwargs['auth'],
                                         bucket=None,
-                                        collection=None)
+                                        collection=None,
+                                        retry=0,
+                                        retry_after=None,
+                                        ignore_batch_4xx=False)
 
         cert_arguments = {
             'bucket': kwargs['certificates_bucket'],
@@ -890,7 +1297,7 @@ class TestMain(unittest.TestCase):
                 self.assertNotEqual(len(out_file.getvalue()), 0)
 
     def test_get_records_read_fails(self):
-        with mock.patch('kinto_client.cli_utils.Client') as MockedClient:
+        with mock.patch('kinto_http.cli_utils.Client') as MockedClient:
             MockedClient.return_value.get_records.side_effect = Exception
             with mock.patch('amo2kinto.exporter.logger') as mocked_logger:
                 exporter.main([])
@@ -905,3 +1312,14 @@ class TestMain(unittest.TestCase):
             with mock.patch('amo2kinto.exporter.open', return_value=out_file):
                 exporter.main(['--out', 'file'])
         self.assertIn('lastupdate="1448372434324"', out_file.getvalue())
+
+    def test_no_certs_post_fx58(self):
+        self.MockedClient.return_value.get_records.side_effect = (
+            [ADDONS_DATA], [PLUGIN_DATA], [GFX_DATA], [CERTIFICATE_DATA]
+        )
+        out_file = StringIO()
+        with mock.patch.object(out_file, 'close'):
+            with mock.patch('amo2kinto.exporter.open', return_value=out_file):
+                exporter.main(['--out', 'file', '--app', constants.FIREFOX_APPID,
+                               '--app-version', '58.0'])
+        self.assertNotIn('<certItems>', out_file.getvalue())
